@@ -12,6 +12,9 @@ use std::{
     mem,
 };
 
+#[cfg(feature = "impl-serde")]
+use std::{any::type_name, fmt, marker::PhantomData};
+
 /// Map of coordinates in a plane to arbitrary data. Optimized given the fact
 /// the coordinates/vectors are in the plane. Keys of the map are `Vec2<K>`.
 #[derive(Debug, Clone)]
@@ -20,6 +23,7 @@ where
     K: Ord,
 {
     neighbours: Vec2<BTreeMap<K, BTreeMap<K, V>>>,
+    len: usize,
 }
 
 impl<K, V> Default for Map<K, V>
@@ -37,7 +41,7 @@ where
 {
     /// Creates a new empty coordinate map.
     pub fn new() -> Self {
-        Map { neighbours: Vec2::from_axes(|_| BTreeMap::new()) }
+        Map { neighbours: Vec2::from_axes(|_| BTreeMap::new()), len: 0 }
     }
 
     /// Returns whether the map is empty.
@@ -45,6 +49,12 @@ where
         self.neighbours.x.is_empty()
     }
 
+    /// Returns the length of the map, i.e. how many [`Vec2`] (keys) are stored
+    /// in this map.
+    pub fn len(&self) -> usize {
+        self.len
+    }
+
     /// Attempts to get the data associated with the given point.
     pub fn get<Q>(&self, point: Vec2<&Q>) -> Option<&V>
     where
@@ -184,6 +194,11 @@ where
             .zip_with(inner_tables, |(key, value), table| {
                 table.insert(key, value)
             });
+
+        if old.x.is_none() {
+            self.len += 1;
+        }
+
         old.x
     }
 
@@ -196,7 +211,7 @@ where
     {
         let values = Vec2 { x: value.clone(), y: value };
         let entries = (!point.clone()).zip(values);
-        let result = self
+        let created = self
             .neighbours
             .as_mut()
             .zip_with(point, |table, key| table.entry(key))
@@ -218,7 +233,11 @@ where
                     }
                 },
             });
-        result.x
+
+        if created.x {
+            self.len += 1;
+        }
+        created.x
     }
 
     /// Updates an existing point's entry with the given value. The entry must
@@ -265,12 +284,16 @@ where
                 let inner_table = table.get_mut(key)?;
 
                 let removed = inner_table.remove(inner_key);
-                if table.is_empty() {
+                if inner_table.is_empty() {
                     table.remove(&key);
                 }
                 removed
             },
         );
+
+        if removed.x.is_some() {
+            self.len -= 1;
+        }
         removed.x
     }
 
@@ -342,6 +365,84 @@ where
     }
 }
 
+#[cfg(feature = "impl-serde")]
+impl<K, V> serde::Serialize for Map<K, V>
+where
+    K: serde::Serialize + Ord,
+    V: serde::Serialize,
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.collect_map(self.rows())
+    }
+}
+
+#[cfg(feature = "impl-serde")]
+#[derive(Debug, Clone)]
+struct DeVisitor<K, V>
+where
+    K: Ord,
+{
+    marker: PhantomData<fn() -> Map<K, V>>,
+}
+
+#[cfg(feature = "impl-serde")]
+impl<K, V> DeVisitor<K, V>
+where
+    K: Ord,
+{
+    fn new() -> Self {
+        Self { marker: PhantomData }
+    }
+}
+
+#[cfg(feature = "impl-serde")]
+impl<'de, K, V> serde::de::Visitor<'de> for DeVisitor<K, V>
+where
+    K: serde::Deserialize<'de> + Ord + Clone,
+    V: serde::Deserialize<'de> + Clone,
+{
+    type Value = Map<K, V>;
+
+    fn expecting(&self, formatter: &mut std::fmt::Formatter) -> fmt::Result {
+        write!(
+            formatter,
+            "Expecting a map from vectors {} to {}",
+            type_name::<Vec2<K>>(),
+            type_name::<V>()
+        )
+    }
+
+    fn visit_map<A>(self, mut access: A) -> Result<Self::Value, A::Error>
+    where
+        A: serde::de::MapAccess<'de>,
+    {
+        let mut map = Map::new();
+
+        while let Some((point, value)) = access.next_entry()? {
+            map.insert(point, value);
+        }
+
+        Ok(map)
+    }
+}
+
+#[cfg(feature = "impl-serde")]
+impl<'de, K, V> serde::Deserialize<'de> for Map<K, V>
+where
+    K: serde::Deserialize<'de> + Ord + Clone,
+    V: serde::Deserialize<'de> + Clone,
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        deserializer.deserialize_map(DeVisitor::new())
+    }
+}
+
 /// Iterator over the neighbours of a point in a map. See [`Map::neighbours`]
 /// and [`Map::neighbours_incl`].
 #[derive(Debug, Clone)]
